@@ -302,6 +302,12 @@ class VersionedObject(object):
                                   '<?>'))
                       for name, field in sorted(self.fields.items())]))
 
+    def __contains__(self, name):
+        try:
+            return self.obj_attr_is_set(name)
+        except AttributeError:
+            return False
+
     @classmethod
     def obj_name(cls):
         """Return the object's name
@@ -502,7 +508,7 @@ class VersionedObject(object):
         finally:
             delattr(self, '_obj_version_manifest')
 
-    def obj_to_primitive(self, target_version=None):
+    def obj_to_primitive(self, target_version=None, version_manifest=None):
         """Simple base-case dehydration.
 
         This calls to_primitive() for each item in fields.
@@ -517,8 +523,16 @@ class VersionedObject(object):
             if self.obj_attr_is_set(name):
                 primitive[name] = field.to_primitive(self, name,
                                                      getattr(self, name))
-        if target_version != self.VERSION:
-            self.obj_make_compatible(primitive, target_version)
+        # NOTE(danms): If we know we're being asked for a different version,
+        # then do the compat step. However, even if we think we're not,
+        # we may have sub-objects that need it, so if we have a manifest we
+        # have to traverse this object just in case. Previously, we
+        # required a parent version bump for any child, so the target
+        # check was enough.
+        if target_version != self.VERSION or version_manifest:
+            self.obj_make_compatible_from_manifest(primitive,
+                                                   target_version,
+                                                   version_manifest)
         obj = {self._obj_primitive_key('name'): self.obj_name(),
                self._obj_primitive_key('namespace'): (
                    self.OBJ_PROJECT_NAMESPACE),
@@ -699,12 +713,6 @@ class VersionedObjectDictCompat(object):
     def __setitem__(self, name, value):
         setattr(self, name, value)
 
-    def __contains__(self, name):
-        try:
-            return self.obj_attr_is_set(name)
-        except AttributeError:
-            return False
-
     def get(self, key, value=_NotSpecifiedSentinel):
         if key not in self.obj_fields:
             raise AttributeError("'%s' object has no attribute '%s'" % (
@@ -765,15 +773,20 @@ class ObjectListBase(collections.Sequence):
         # obj_relationships
         if self.child_versions:
             relationships = self.child_versions.items()
-        elif self.obj_relationships:
-            relationships = self._obj_relationship_for('objects',
-                                                       target_version)
+        else:
+            try:
+                relationships = self._obj_relationship_for('objects',
+                                                           target_version)
+            except exception.ObjectActionError:
+                # No relationship for this found in manifest or
+                # in obj_relationships
+                relationships = {}
 
         try:
-            # NOTE(rlrossit): If child_versions and obj_relationships weren't
-            # set, just backport to child version 1.0 (maintaining default
+            # NOTE(rlrossit): If we have no version information, just
+            # backport to child version 1.0 (maintaining default
             # behavior)
-            if self.child_versions or self.obj_relationships:
+            if relationships:
                 _get_subobject_version(target_version, relationships,
                                        lambda ver: _do_subobject_backport(
                                            ver, self, 'objects', primitive))
